:- module array_utils.
:- interface.
:- import_module array, int.

:- func map_with_index(func(int, T1) = T2, array(T1)) = array(T2).
:- mode map_with_index(func(in, in) = out is det, in) = array_uo is det.

:- pred map_fold_with_index(pred(int, T1, T2, Acc, Acc),
	array(T1), array(T2), Acc, Acc).
:- mode map_fold_with_index(pred(in, in, out, in, out) is det,
	in, array_uo, in, out) is det.

%-----------------------------------------------------------------------------%

:- implementation.

map_with_index(Func, OldArray) = NewArray :-
	( array__semidet_lookup(OldArray, 0, Elem0) ->
		Size = array__size(OldArray),
		Elem = Func(0, Elem0),
		NewArray0 = array__init(Size, Elem),
		NewArray = map_with_index_2(1, Size, Func, OldArray, NewArray0)
	;
		array__make_empty_array(NewArray)
	).

map_fold_with_index(Pred, OldArray, NewArray, Acc0, Acc) :-
	( array__semidet_lookup(OldArray, 0, Elem0) ->
		Size = array__size(OldArray),
		Pred(0, Elem0, Elem, Acc0, Acc1),
		NewArray0 = array__init(Size, Elem),
		map_fold_with_index_2(1, Size, Pred, OldArray,
			NewArray0, NewArray, Acc1, Acc)
	;
		array__make_empty_array(NewArray),
		Acc0 = Acc
	).

:- func map_with_index_2(int, int, func(int, T1) = T2, array(T1), array(T2)) =
	array(T2).
:- mode map_with_index_2(in, in, (func(in, in) = out is det), in, array_di) =
	array_uo is det.

map_with_index_2(N, Size, Func, OldArray, NewArray0) = NewArray :-
	( N >= Size ->
		NewArray = NewArray0
	;
		array__lookup(OldArray, N, OldElem),
		NewElem = Func(N, OldElem),
		array__set(NewArray0, N, NewElem, NewArray1),
		NewArray = map_with_index_2(N + 1, Size, Func, OldArray,
			NewArray1)
	).

:- pred map_fold_with_index_2(int, int, pred(int, T1, T2, Acc, Acc),
		array(T1), array(T2), array(T2), Acc, Acc).
:- mode map_fold_with_index_2(in, in, (pred(in, in, out, in, out) is det),
		in, array_di, array_uo, in, out) is det.

map_fold_with_index_2(N, Size, Pred, OldArray, NewArray0, NewArray,
		Acc0, Acc) :-
	( N >= Size ->
		NewArray = NewArray0,
		Acc = Acc0
	;
		array__lookup(OldArray, N, OldElem),
		Pred(N, OldElem, NewElem, Acc0, Acc1),
		array__set(NewArray0, N, NewElem, NewArray1),
		map_fold_with_index_2(N + 1, Size, Pred, OldArray,
			NewArray1, NewArray, Acc1, Acc)
	).

%-----------------------------------------------------------------------------%
