:- module output.
:- interface.
:- import_module char.
:- use_module curs.
:- use_module io.

% output routines that go either to stderr or to curses,
% depending on whether the --curses option is set.

:- pred output_string(curs__attr, string, io__state, io__state).
:- mode output_string(in, in, di, uo) is det.

:- pred output_string(string, io__state, io__state).
:- mode output_string(in, di, uo) is det.

:- pred output_char(curs__attr, char, io__state, io__state).
:- mode output_char(in, in, di, uo) is det.

:- pred output_char(char, io__state, io__state).
:- mode output_char(in, di, uo) is det.

:- pred nl(io__state, io__state).
:- mode nl(di, uo) is det.

:- pred maybe_curses(pred(io__state, io__state), io__state, io__state).
:- mode maybe_curses(pred(di, uo) is det, di, uo) is det.

:- implementation.
:- import_module globals, options, getopt, bool.

output_string(S) -->
	output_string(curs__normal, S).

output_string(Attr, S) -->
	globals__io_lookup_bool_option(curses, Curses),
	( { Curses = yes } ->
		curs__addstr(Attr, S)
	;
		io__stderr_stream(Stderr),
		io__write_string(Stderr, S)
	).

output_char(S) -->
	output_char(curs__normal, S).

output_char(Attr, C) -->
	globals__io_lookup_bool_option(curses, Curses),
	( { Curses = yes } ->
		curs__addch(Attr, char__to_int(C))
	;
		io__stderr_stream(Stderr),
		io__write_char(Stderr, C)
	).

nl -->
	globals__io_lookup_bool_option(curses, Curses),
	( { Curses = yes } ->
		output_string("\n\r")
	;
		output_string("\n")
	).

maybe_curses(P) -->
	globals__io_lookup_bool_option(curses, Curses),
	( { Curses = yes } ->
		P
	;
		[]
	).

